/**
* $Header: /home/cvs/mediabox404/scheduler404/src/xml_config.c,v 1.3 2005/07/18 18:46:54 bedlam Exp $
* @package MediaBox404_prg
* @copyright (C) 2004 - 2005 erreur404.org
* @license http://www.gnu.org/copyleft/gpl.html GNU/GPL
* MediaBox404 is Free Software
* @author vortex@erreur404.org
* $Id: xml_config.c,v 1.3 2005/07/18 18:46:54 bedlam Exp $
*/


#include "xml_config.h"
#include "scheduler404.h"
#include "log.h"
#include "util.h"

#include <libxml/parser.h>
#include <libxml/xmlmemory.h>
#include <string.h>

/* Private function declarations */
static int parse_file (const char *configfile, mediabox404_config_t *mediabox404_config);
static void parse_sql_node (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur, mediabox404_config_t *mediabox404_config);
static void parse_timing_node (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur,mediabox404_config_t *mediabox404_config);
static void parse_log_node (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur,mediabox404_config_t *mediabox404_config);
static char* mediabox404_xml_read_node (xmlDocPtr doc, xmlNodePtr node);

/* Top level XML configfile parser */
int
mediabox404_xml_parse_config_file (mediabox404_config_t *mediabox404_config, const char *configfile)
{

 if (!mediabox404_util_verify_file (configfile)) {
    mediabox404_log("XML Parser Error: Could not open configfile. [%s]", configfile);
    return 0;
  }

  /* Setup the parser options */
  LIBXML_TEST_VERSION
  xmlKeepBlanksDefault(0);
  /* Parse the file and be happy */
  return parse_file (configfile, mediabox404_config);
}

/* I hope you can tell this is my first try at xml and libxml :)  */
static int
parse_file (const char *configfile, mediabox404_config_t *mediabox404_config)
{
  xmlDocPtr doc = NULL;
  xmlNsPtr ns = NULL;
  xmlNodePtr cur = NULL;

  /* This does the actual parsing */
  if (!(doc = xmlParseFile (configfile))) {
    mediabox404_log("XML Parser: Error while parsing %s", configfile);
    return 0;
  }
	
  /* Gimme the root element dammit! */
  if (!(cur = xmlDocGetRootElement (doc))) {
    mediabox404_log("XML Parser: Empty document %s", configfile);
    xmlFreeDoc (doc);
    return 0;
  }
	
  /* First element should be configuration */
  if (!cur->name || (strcmp (cur->name, "Configuration"))) {
    mediabox404_log("XML Parser: Document of invalid type, root node is not 'Configuration'");
    xmlFreeDoc (doc);
    return 0;
  }

  /* Get the configuration tree */
  /* Tree traversal */
  cur = cur->xmlChildrenNode;

#ifdef HAVE_LIBXML_PARSER_H
  while (cur && xmlIsBlankNode (cur)) {
    cur = cur->next;
  }
#endif

  if (cur == NULL) {
    /* Right type of document, but no configuration, guess that is ok */
    xmlFreeDoc (doc);
    return 1;
  }

  /* Separate the parsing into different submodules,
   * Server, Stream, Playlist and Execution */
  for (; cur != NULL; cur = cur->next) {
    if (cur->type == XML_COMMENT_NODE)
      continue;
    if (strcmp (cur->name, "Sql") == 0) {
      parse_sql_node (doc, ns, cur->xmlChildrenNode, mediabox404_config);
    } else if (strcmp (cur->name, "Timing") == 0) {
      parse_timing_node (doc, ns, cur->xmlChildrenNode, mediabox404_config);
    } else if (strcmp (cur->name, "Log") == 0) {
      parse_log_node (doc, ns, cur->xmlChildrenNode, mediabox404_config);
    } else {
      mediabox404_log("Parse XML Unknown Node: %s", cur->name);
    }
  }
	
  /* Be a good boy and cleanup */
  xmlFreeDoc (doc);
  return 1;
}

static void parse_sql_node (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur,
		      mediabox404_config_t *mediabox404_config)
{
  for (; cur; cur = cur->next) {
    if (cur->type == XML_COMMENT_NODE)
      continue;

    if (strcmp (cur->name, "Hostname") == 0) {
      mediabox404_util_free (mediabox404_config->sql.hostname);
      mediabox404_config->sql.hostname = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Port") == 0) {
      mediabox404_config->sql.port = atoi (mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Unixsocket") == 0) {
      mediabox404_util_free (mediabox404_config->sql.usocket);
      mediabox404_config->sql.usocket = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "User") == 0) {
      mediabox404_util_free (mediabox404_config->sql.user);
      mediabox404_config->sql.user = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Password") == 0) {
      mediabox404_util_free (mediabox404_config->sql.password);
      mediabox404_config->sql.password = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Database") == 0) {
      mediabox404_util_free (mediabox404_config->sql.database);
      mediabox404_config->sql.database = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Streamstack") == 0) {
      mediabox404_util_free (mediabox404_config->sql.streamstack);
      mediabox404_config->sql.streamstack = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Progstack") == 0) {
      mediabox404_util_free (mediabox404_config->sql.progstack);
      mediabox404_config->sql.progstack = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Lastplaylist") == 0) {
      mediabox404_util_free (mediabox404_config->sql.lastplaylist);
      mediabox404_config->sql.lastplaylist = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Pilotstack") == 0) {
      mediabox404_util_free (mediabox404_config->sql.pilotstack);
      mediabox404_config->sql.pilotstack = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Log") == 0) {
      mediabox404_util_free (mediabox404_config->sql.log);
      mediabox404_config->sql.log = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Tempstack") == 0) {
      mediabox404_util_free (mediabox404_config->sql.tempstack);
      mediabox404_config->sql.tempstack = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    
    } else if (strcmp (cur->name, "Datapath") == 0) {
      mediabox404_util_free (mediabox404_config->sql.datapath);
      mediabox404_config->sql.datapath = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Data") == 0) {
      mediabox404_util_free (mediabox404_config->sql.data);
      mediabox404_config->sql.data = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Playlistdata") == 0) {
      mediabox404_util_free (mediabox404_config->sql.playlistdata);
      mediabox404_config->sql.playlistdata = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Lastmainplaylist") == 0) {
      mediabox404_util_free (mediabox404_config->sql.lastmainplaylist);
      mediabox404_config->sql.lastmainplaylist = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Lastpromo") == 0) {
      mediabox404_util_free (mediabox404_config->sql.lastpromo);
      mediabox404_config->sql.lastpromo = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Lastjingle") == 0) {
      mediabox404_util_free (mediabox404_config->sql.lastjingle);
      mediabox404_config->sql.lastjingle = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Programme") == 0) {
      mediabox404_util_free (mediabox404_config->sql.programme);
      mediabox404_config->sql.programme = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Currentplaylisttype") == 0) {
      mediabox404_util_free (mediabox404_config->sql.currentplaylisttype);
      mediabox404_config->sql.currentplaylisttype = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Playlist") == 0) {
      mediabox404_util_free (mediabox404_config->sql.playlist);
      mediabox404_config->sql.playlist = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Live") == 0) {
      mediabox404_util_free (mediabox404_config->sql.live);
      mediabox404_config->sql.live = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Metadata") == 0) {
      mediabox404_util_free (mediabox404_config->sql.metadata);
      mediabox404_config->sql.metadata = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Streamlog") == 0) {
      mediabox404_util_free (mediabox404_config->sql.streamlog);
      mediabox404_config->sql.streamlog = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "Mysql_options") == 0) {
      mediabox404_util_free (mediabox404_config->sql.mysql_options);
      mediabox404_config->sql.mysql_options = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else {
      printf("parse XML SQL node Unknown keyword: %s\n", cur->name);
    }
  }
}

static void
parse_log_node (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur,
				mediabox404_config_t *mediabox404_config)
{
  for (; cur; cur = cur->next) {
    if (cur->type == XML_COMMENT_NODE)
      continue;

    if (strcmp (cur->name, "Verbose") == 0) {
      mediabox404_config->verbose = atoi (mediabox404_xml_read_node (doc, cur));
    }  else if (strcmp (cur->name, "BaseDirectory") == 0) {
      mediabox404_util_free (mediabox404_config->base_directory);
      mediabox404_config->base_directory = mediabox404_util_strdup(mediabox404_xml_read_node (doc, cur));
    } else {
      mediabox404_log("parse XML Log node Unknown keyword: %s", cur->name);
    }
  }
}

static void
parse_timing_node (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur,
				mediabox404_config_t *mediabox404_config)
{
  for (; cur; cur = cur->next) {
    if (cur->type == XML_COMMENT_NODE)
      continue;

    if (strcmp (cur->name, "jingle") == 0) {
      mediabox404_config->jingle_timing = atoi (mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "promo") == 0) {
      mediabox404_config->promo_timing = atoi (mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "program") == 0) {
      mediabox404_config->program_timing = atoi (mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "period_control") == 0) {
      mediabox404_config->period_control = atoi (mediabox404_xml_read_node (doc, cur));
    } else if (strcmp (cur->name, "max_control") == 0) {
      mediabox404_config->max_control = atoi (mediabox404_xml_read_node (doc, cur));
    } else {
      mediabox404_log("parse XML timing node Unknown keyword: %s", cur->name);
    }
  }
}

static char* mediabox404_xml_read_node (xmlDocPtr doc, xmlNodePtr node)
{
  return (char *)xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
}
