/* $Id: cue.c,v 1.1.1.1 2005/05/28 17:24:24 bedlam Exp $ */
/* cue.c
 * - Functions for cue file in ices
 * Copyright (c) 2000 Alexander Havng
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#include "definitions.h"
#include "metadata.h"

extern stream404_config_t stream404_config;

char *stream404_cue_filename = NULL;
static int stream404_cue_lineno = 0;

/* Syntax of cue file
 * filename
 * size
 * bitrate
 * minutes:seconds total
 * % played
 * current line in playlist
 * artist
 * songname
 */

/* Global function definitions */

/* Update the cue file with a set of variables */
void
stream404_cue_update (input_stream_t* source) 
{
  char buf[1024];
  char artist[1024];
  char title[1024];
  FILE *fp = stream404_util_fopen_for_writing (stream404_cue_get_filename());
  
  if (!fp) {
    stream404_log ("Could not open cuefile [%s] for writing, cuefile not updated!", stream404_cue_get_filename ());
    return;
  }

  artist[0] = '\0';
  title[0] = '\0';
  stream404_metadata_get (artist, sizeof (artist), title, sizeof (title));

  fprintf (fp, "%s\n%d\n%d\n%s\n%f\n%d\n%s\n%s\n", source->path,
	   (int)source->filesize, source->bitrate,
       stream404_util_file_time (source->bitrate, source->filesize, buf),
	   stream404_util_percent (source->bytes_read, source->filesize),
	   stream404_cue_lineno, artist, title);

  stream404_util_fclose (fp);
}

/* Cleanup the cue module by removing the cue file */
void
stream404_cue_shutdown (void)
{
	const char *filename = stream404_cue_get_filename ();

	if (filename && filename[0])
		remove (filename);
}

void
stream404_cue_set_lineno (int lineno)
{
  stream404_cue_lineno = lineno;
}

/* Mutator for the cue filename */
void
stream404_cue_set_filename (const char *filename)
{
	stream404_cue_filename = stream404_util_strdup (filename);
}

/* Return the current cue filename, and create it if
 * necessary */
const char *
stream404_cue_get_filename (void)
{
  static char buf[1024];

  if (stream404_cue_filename)
    return stream404_cue_filename;

  if (! stream404_config.base_directory) {
    stream404_log_error ("Base directory is invalid");
    return NULL;
  }

  snprintf (buf, sizeof (buf), "%s/stream404.cue", stream404_config.base_directory);

  return buf;
}


			
