/* $Id: reencode.h,v 1.1.1.1 2005/05/28 17:24:30 bedlam Exp $ */
/* reencode.h
 * - Function declarations for reencoding
 * Copyright (c) 2000 Alexander Havng
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

/* Public function declarations */
void stream404_reencode_initialize (void);
void stream404_reencode_shutdown (void);
void stream404_reencode_reset (input_stream_t* source);
int stream404_reencode_decode (unsigned char* buf, size_t blen, size_t olen,
			  int16_t* left, int16_t* right);
int stream404_reencode (stream404_stream_t* stream, int nsamples, int16_t* left,
		   int16_t* right, unsigned char *outbuf, int outlen);
int stream404_reencode_flush (stream404_stream_t* stream, unsigned char *outbuf,
			 int maxlen);
void vorbis_reencode_reset (input_stream_t *, stream404_stream_t *);

void vorbis_encode_data_float(vorbis_encoder_t *s, float **pcm, int samples);
void vorbis_encode_data(vorbis_encoder_t *s, signed char *buf, int bytes, int bigendian);
int vorbis_encode_dataout(vorbis_encoder_t *s, ogg_page *og);
void vorbis_encode_clear(vorbis_encoder_t *s);
int vorbis_encode_flush(vorbis_encoder_t *s, ogg_page *og);
void vorbis_encode_finish(vorbis_encoder_t *s);

typedef float SAMPLE;

typedef struct
{
  unsigned int channels, infreq, outfreq, taps;
  float *table;
  SAMPLE *pool;
  
  /* dynamic bits */
  int poolfill;
  int offset;
} resampler_state_t;

typedef struct {
    float *buffer;
    int buflen;
} downmix_state_t;

typedef struct {
  resampler_state_t resampler;
  int channels;
  
  float **buffers;
  int buffill;
  int bufsize;
  
  float **convbuf;
  int convbuflen;
} resample_state_t;


/*--------------------------------- resampling part -------------------------*/



typedef enum
{
    RES_END,
    RES_GAIN,    /* (double)1.0 */
    RES_CUTOFF,    /* (double)0.80 */ 
    RES_TAPS,    /* (int)45 */
    RES_BETA    /* (double)16.0 */
} resampler_parameter;

int resampler_init(resampler_state_t *state, int channels, int outfreq, int infreq, resampler_parameter op1, ...);
/*
 * Configure *state to manage a data stream with the specified parameters.  The
 * string 'params' is currently unspecified, but will configure the parameters
 * of the filter.
 *
 * This function allocates memory, and requires that resampler_clear() be called when
 * the buffer is no longer needed.
 *
 *
 * All counts/lengths used in the following functions consider only the data in
 * a single channel, and in numbers of samples rather than bytes, even though
 * functionality will be mirrored across as many channels as specified here.
 */


int resampler_push_max_input(resampler_state_t const *state, size_t maxoutput);
/*
 *  Returns the maximum number of input elements that may be provided without
 *  risk of flooding an output buffer of size maxoutput.  maxoutput is
 *  specified in counts of elements, NOT in bytes.
 */


int resampler_push_check(resampler_state_t const *state, size_t srclen);
/*
 * Returns the number of elements that will be returned if the given srclen
 * is used in the next call to resampler_push().
 */


int resampler_push(resampler_state_t *state, SAMPLE **dstlist, SAMPLE const **srclist, size_t srclen);
int resampler_push_interleaved(resampler_state_t *state, SAMPLE *dest, SAMPLE const *source, size_t srclen);
/*
 * Pushes srclen samples into the front end of the filter, and returns the
 * number of resulting samples.
 *
 * resampler_push(): srclist and dstlist point to lists of pointers, each of which
 * indicates the beginning of a list of samples.
 *
 * resampler_push_interleaved(): source and dest point to the beginning of a list of
 * interleaved samples.
 */


int resampler_drain(resampler_state_t *state, SAMPLE **dstlist);
int resampler_drain_interleaved(resampler_state_t *state, SAMPLE *dest);
/*
 * Recover the remaining elements by flushing the internal pool with 0 values,
 * and storing the resulting samples.
 *
 * After either of these functions are called, *state should only re-used in a
 * final call to resampler_clear().
 */


void resampler_clear(resampler_state_t *state);
/*
 * Free allocated buffers, etc.
 */


/*----------------------------- audio.c ------------------------------------*/


downmix_state_t *downmix_initialise(void);
void downmix_clear(downmix_state_t *s);
void downmix_buffer(downmix_state_t *s, signed char *buf, int len, int be);
void downmix_buffer_float(downmix_state_t *s, float **buf, int samples);

resample_state_t *resample_initialise(int channels, int infreq, int outfreq);
void resample_clear(resample_state_t *s);
void resample_buffer(resample_state_t *s, signed char *buf, int buflen, int be);
void resample_buffer_float(resample_state_t *s, float **buf, int buflen);
void resample_finish(resample_state_t *s);

