/* $Id: stream404types.h,v 1.1.1.1 2005/05/28 17:24:24 bedlam Exp $ */
/* stream404types.h
 * - Datatypes for ices
 * Copyright (c) 2000 Alexander Havng
 * Copyright (c) 2001-4 Brendan Cully <brendan@icecast.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#ifndef _STREAM404_STREAM404TYPES_H
#define _STREAM404_STREAM404TYPES_H

typedef enum {
  icy_protocol_e,
  xaudiocast_protocol_e,
  http_protocol_e
} protocol_t;

typedef enum {
  stream404_playlist_sql_e,
  stream404_playlist_builtin_e,
  stream404_playlist_python_e,
  stream404_playlist_perl_e,
  stream404_playlist_null_e /* == NULL */
} playlist_type_t;

/* -- output stream types -- */
typedef enum {
  STREAM404_OUTPUT_VORBIS,
  STREAM404_OUTPUT_MP3
} output_type_t;

typedef enum {
  STREAM404_INPUT_UNKNOWN,
  STREAM404_INPUT_VORBIS,
  STREAM404_INPUT_MP3,
  STREAM404_INPUT_MP4,
  STREAM404_INPUT_FLAC
} input_type_t;

typedef enum {
  LOCAL,
  REMOTE
} input_location_t;

typedef enum {
  _FILE,
  STREAM
} input_spec_t;


/* from ices-2.0. ogg/vorbis reencode struct */
typedef struct {
  ogg_stream_state os;
  vorbis_block vb;
  vorbis_dsp_state vd;
  vorbis_info vi;
  
  int samples_in_current_page;
  int samplerate;
  ogg_int64_t prevgranulepos;
  int in_header;
} vorbis_encoder_t;


/* EO from ices-2.0.0 */

typedef struct stream404_stream_St {
  shout_t* conn;
  time_t connect_delay;
  int errs;
  void* encoder_state;
  void* resample_state;
  void* downmix_state;
  
  char *host;
  int port;
  char *password;
  protocol_t protocol;

  char* mount;
  char* dumpfile;

  char* name;
  char* genre;
  char* description;
  char* url;
  int ispublic;

  int reencode;
  int bitrate;
  float quality; /* bitrate and quality are mutually exclusive */
  int out_samplerate;
  int out_numchannels;
  
  output_type_t type;

  struct stream404_stream_St* next;
} stream404_stream_t;

typedef struct {
  /* sql configuration */
  char *hostname;
  char *usocket;
  int port;
  char *sqltype;
  char *user;
  char *password;
  char *database; /* database name */
  char *table;   /* current information table (where to get the next file) */
  char *urg_table; /* same as below but used ofr emergency purpose */
  char *currentstream_table; /* current stream table */
  char *streamlog_table; /* log table */
  char *metadata_table; /* to update metadata */
} sql_t;

typedef struct {
  playlist_type_t playlist_type;
  int randomize;
  char* playlist_file;
  char* module;

  char* (*get_next) (void *);     /* caller frees result */
  char* (*get_next_urg) (void *); /* caller frees result */
  char* (*get_metadata) (void); /* caller frees result */
  int (*get_lineno) (void);
  int (*reload) (void);
  void (*shutdown) (void);
} playlist_module_t;


/* -- input stream types -- */
typedef struct _input_stream_t {
  input_type_t type;
  input_spec_t spec;
  input_location_t location;
  
  /* how many reconnects can we tolerate (stream only)*/
  int reconnects;

  char* path;
  int fd;
  size_t filesize;
  size_t bytes_read;
  unsigned int bitrate;
  unsigned int samplerate;
  unsigned int channels;

  void* data;

  ssize_t (*read)(struct _input_stream_t* self, void* buf, size_t len);
  /* len is the size in bytes of left or right. The two buffers must be
   * the same size. */
  ssize_t (*readpcm)(struct _input_stream_t* self, size_t len, int16_t* left,
		     int16_t* right);
  int (*close)(struct _input_stream_t* self);
} input_stream_t;

typedef struct _stream404_plugin {
  const char *name;

  int (*init)(void);
  void (*new_track)(input_stream_t *source);
  int (*process)(int ilen, int16_t *il, int16_t *ir);
  void (*shutdown)(void);

  struct _stream404_plugin *next;
} stream404_plugin_t;

typedef struct {
  int daemon;
  int verbose;
  int reencode;
  char *configfile;
  char *base_directory;
  FILE *logfile;

  int childloop;
  
  sql_t sql;

  stream404_stream_t* streams;
  playlist_module_t pm;
  playlist_module_t pm_backup;
  stream404_plugin_t *plugins;
} stream404_config_t;

#endif
